<?php
/**
 * GiftPay - Payment Module for PrestaShop 1.7
 *
 * Order Validation Controller
 *
 * @author GiftPay
 * @license https://opensource.org/licenses/afl-3.0.php
 */

class GiftpayValidationModuleFrontController extends ModuleFrontController
{
    /**
     * Processes the data sent by the payment form
     */
    public function postProcess()
    {
        /**
         * Get current cart object from session
         */
        $cart = $this->context->cart;
        $authorized = false;

        /**
         * Verify if this module is enabled and if the cart has
         * a valid customer, delivery address and invoice address
         */
        if (!$this->module->active || $cart->id_customer == 0 || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        /**
         * Verify if this payment module is authorized
         */
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'giftpay') {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die($this->l('This payment method is not available.'));
        }

        /** @var CustomerCore $customer */
        $customer = new Customer($cart->id_customer);

        /**
         * Check if this is a vlaid customer account
         */
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        /**
         * Place the order
         */
        $this->module->validateOrder(
            (int) $this->context->cart->id,
            Configuration::get('PS_OS_GP_AWAITING_PAYMENT'),
            (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            null,
            null,
            (int) $this->context->currency->id,
            false,
            $customer->secure_key
        );

        $isSandbox = Configuration::get('SANDBOX_MODE');
        $merchantsCode = $isSandbox ? Configuration::get('SANDBOX_MERCHANTS_CODE') : Configuration::get('MERCHANTS_CODE');

        $args = [
            'website_code' => $merchantsCode,
            'amount' => (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            'user_id' => $cart->id_customer,
            'unique_id' => Order::getIdByCartId($this->context->cart->id)
        ];

        $domain = $isSandbox ? 'sandbox.giftpay.me' : 'giftpay.me';
        $redirect = "https://{$domain}/b/init?". http_build_query($args);

        Tools::redirect($redirect);
    }
}
