<?php
/**
 * GiftPay - Payment Module for PrestaShop 1.7
 *
 * Handle Webhooks
 *
 * @author GiftPay
 * @license https://opensource.org/licenses/afl-3.0.php
 */

class GiftpayWebhookModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        if(isset($_GET['order_id'])) {
            $this->complete($_GET['order_id']);
        }

        $this->setTemplate('module:giftpay/views/templates/front/empty.tpl');
    }

    public function complete($orderId)
    {
        $isSandBox = Configuration::get('SANDBOX_MODE');
        $apiMerchantsCode =  $isSandBox ? Configuration::get('SANDBOX_MERCHANTS_CODE') : Configuration::get('MERCHANTS_CODE');
        $apiSecretToken = $isSandBox ? Configuration::get('SANDBOX_SECRET_TOKEN') : Configuration::get('SECRET_TOKEN');

        $url = 'https://' . ($isSandBox ? 'sandbox-api.giftpay.me' : 'api.giftpay.me') . '/website-api/get-order-by-order-id';
        $headers = ["SECRET-TOKEN: $apiSecretToken"];
        $params = [
            'order_id' => $orderId,
            'website_code' => $apiMerchantsCode
        ];

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $response = curl_exec($ch);


        if (curl_errno($ch)) {
            return;
        }
        curl_close($ch);

        $body = json_decode( $response, true );
        $order = new Order((int)$body['unique_id']);

        if  ($order) {
            $refundedAmount = $this->createCompletedTransactionsAndGetRefundedAmount($order, $body['transactions']);

            if ($refundedAmount > 0 && $body['amount_requested'] > $body['amount_paid'] - $refundedAmount) {
                $this->changeOrderState($order, Configuration::get('PS_OS_REFUND'));
            }
            elseif ($body['amount_requested'] <= $body['amount_paid']) {
                $this->changeOrderState($order, Configuration::get('PS_OS_GP_AWAITING_FULL_PAYMENT'));
            }
            else {
                $this->changeOrderState($order, Configuration::get('PS_OS_GP_AWAITING_FULL_PAYMENT'));
            }
        }
    }

    private function changeOrderState(Order $order, $stateId)
    {
        $orderState = new OrderState($stateId);

        $current_order_state = $order->getCurrentOrderState();
        if ($current_order_state->id != $orderState->id) {
            // Create new OrderHistory
            $history = new OrderHistory();
            $history->id_order = $order->id;
            $useExistingsPayment = false;

            if (!$order->hasInvoice()) {
                $useExistingsPayment = true;
            }

            $history->changeIdOrderState((int)$orderState->id, $order, $useExistingsPayment);
        }
    }

    private function createCompletedTransactionsAndGetRefundedAmount(Order $order, $transactions)
    {
        $isSandbox = Configuration::get('SANDBOX_MODE');
        $refundedAmount = 0;

        $orderTransactions = Db::getInstance()->executeS('SELECT transaction_id FROM ' . _DB_PREFIX_ . 'order_payment;');
        $transactionIds = [];


        foreach ($orderTransactions as $orderTransaction) {
            $transactionIds[] = $orderTransaction['transaction_id'];
        }

        foreach ($transactions as $transaction) {
            $transactionId = $transaction['txn_id'] . '-' . ($isSandbox ? '1' : '0');

            if ($transaction['status'] == 'rejected') {
                $refundedAmount += $transaction['amount'];
            }

            if (!in_array($transactionId, $transactionIds)) {
                $status = $order->addOrderPayment($transaction['amount'], $this->module->displayName, $transactionId);
                echo ' ID: ' . $transactionId . ' status: ' . $status;
            }
        }

        return $refundedAmount;
    }
}
